﻿// -*- C++ -*-
/*!
 * @file  KinectFaceTest.cpp
 * @brief Image on Kinect
 * @date $Date$
 *
 * @author 名城大学
 * ロボットシステムデザイン研究室
 *
 * $Id$
 */

#include "KinectFaceTest.h"

// Module specification
// <rtc-template block="module_spec">
static const char* kinectface_spec[] =
  {
    "implementation_id", "KinectFaceTest",
    "type_name",         "KinectFaceTest",
    "description",       "Image on Kinect",
    "version",           "1.0.0",
    "vendor",            "rsdlab",
    "category",          "Sensor",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.Output", "raw,camera,jpeg",
    "conf.default.Size", "480x270",

    // Widget
    "conf.__widget__.Output", "checkbox",
    "conf.__widget__.Size", "text",
    // Constraints
    "conf.__constraints__.Output", "(raw,camera,jpeg)",
    "conf.__constraints__.Size", "(240x135,480x270,960x540,1920x1080)",

    "conf.__type__.Output", "string",
    "conf.__type__.Size", "string",

    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
KinectFaceTest::KinectFaceTest(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_CameraImageOut("CameraImage", m_CameraImage),
    m_FaceAngleOut("FaceAngle", m_FaceAngle),
    m_FacepropertyOut("Faceproperty", m_Faceproperty)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
KinectFaceTest::~KinectFaceTest()
{
}


/*!
 * Kinectへ接続する
 */
RTC::ReturnCode_t KinectFaceTest::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("CameraImage", m_CameraImageIn);
  addInPort("FaceAngle", m_FaceAngleIn);
  addInPort("Faceproperty", m_FacepropertyIn);
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("Output", m_Output, "raw,camera,jpeg");
  bindParameter("Size", m_Size, "480x270");
  // </rtc-template>
  
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t KinectFaceTest::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t KinectFaceTest::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t KinectFaceTest::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * ポート及びデータの初期化
 */

RTC::ReturnCode_t KinectFaceTest::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}

/*!
 * データの破棄
 */

RTC::ReturnCode_t KinectFaceTest::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}

/*!
 * 画像データ出力
 */

RTC::ReturnCode_t KinectFaceTest::onExecute(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t KinectFaceTest::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t KinectFaceTest::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * Kinectへの再接続を試みる
 */

RTC::ReturnCode_t KinectFaceTest::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t KinectFaceTest::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t KinectFaceTest::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void KinectFaceTestInit(RTC::Manager* manager)
  {
    coil::Properties profile(kinectface_spec);
    manager->registerFactory(profile,
                             RTC::Create<KinectFaceTest>,
                             RTC::Delete<KinectFaceTest>);
  }
  
};


