﻿#ifndef TROBOT_SENSOR_DATA_HPP
#define TROBOT_SENSOR_DATA_HPP

#include <vector> 
#include <cstddef>

namespace TRobot
{
  ////////////////////////////////////////////////////////////////////////////////
  /*!
   *  @brief メイン基板に接続されたセンサのデータを集約する構造体
   *
   *  @copybrief
   */
  struct BaseSensorData
  {
    BaseSensorData();
    static const size_t NUM_ENC_COUNTS = 16; //!< データチャンク1個あたりのエンコーダ値のサンプルの個数
    static const size_t NUM_XYZ = 3; //!< XYZ軸の軸の数
    int m_leftEnc[NUM_ENC_COUNTS];  //!< 左モータのエンコーダ値
    int m_rightEnc[NUM_ENC_COUNTS]; //!< 右モータのエンコーダ値
    int m_cycleCount[NUM_ENC_COUNTS]; //!< サイクルカウント
    int m_acc[NUM_XYZ]; //!< 加速度センサのデータ (インデックスと各軸の対応: 0:X, 1:Y, 2:Z) 
    int m_gyr[NUM_XYZ]; //!< ジャイロのデータ (インデックスと各軸の対応: 0:X, 1:Y, 2:Z) 
    int m_mag[NUM_XYZ]; //!< 磁気センサのデータ (インデックスと各軸の対応: 0:X, 1:Y, 2:Z) 
    static int const DEFAULT_VAL = 0; //!< センサデータのデフォルト値 (通信失敗の場合などに利用)
  };
} // namespace TRobot
  
#endif // TROBOT_SENSOR_DATA_HPP
