﻿#include "TRobotSerialIOIface.hpp"
#include "TRobotSerialIOWindows.hpp"
#include "TRobotSensorData.hpp"
#include <iostream>
#include <cstdlib>
#include <windows.h>


static void checkArgs(int argc, char *argv[])
{
  using namespace std;
  
  if(argc < 2){
    cerr << "no port specified" << endl;
    cerr << "  Example: " << argv[0] << " COM10 " << endl;
    exit(EXIT_FAILURE);
  }
}

int main(int argc, char *argv[])
{
  using namespace std;
  TRobot::SerialIOIface *serIO = new TRobot::SerialIOWindows();
  const DWORD PERIOD = 50; // [msec]
  TRobot::BaseSensorData sdata;
  TRobot::RetCode retVal;
  int leftRotVel  = 0;
  int rightRotVel = 0;

  checkArgs(argc, argv);
  
  retVal = serIO->start(argv[1]);
  if(retVal != TRobot::SUCCESS){
    exit(EXIT_FAILURE);
  }
  cout << "input vel (e.g. 50 50): " << endl;
  cin >> leftRotVel >> rightRotVel;
  cout << "lrv = " << leftRotVel << " rrv = " << rightRotVel << endl;
  //serIO->sendWheelRotVel(leftRotVel, rightRotVel);
  
  while(true){
    serIO->sendWheelRotVel(leftRotVel, rightRotVel);

    while(true){
      retVal = serIO->receiveSensorData(sdata);
      if(retVal == TRobot::SUCCESS){
        for(size_t i = 0; i < TRobot::BaseSensorData::NUM_ENC_COUNTS; i++){
          cout << sdata.m_leftEnc[i] << " " << sdata.m_rightEnc[i] << " "; 
        }
        for(size_t i = 0; i < TRobot::BaseSensorData::NUM_XYZ; i++){
          cout << sdata.m_acc[i] << " ";
        }
        for(size_t i = 0; i < TRobot::BaseSensorData::NUM_XYZ; i++){
          cout << sdata.m_gyr[i] << " ";
        }
        for(size_t i = 0; i < TRobot::BaseSensorData::NUM_XYZ; i++){
          cout << sdata.m_mag[i] << " ";
        }
        cout << endl;
      
      }else{
        // cerr << "Error on receiving data" << endl;
        break;
      }
    }
    
    Sleep(PERIOD);
  }

  retVal = serIO->stop();

  delete serIO;
}
